@@ -9,6 +9,7 @@
 #include <endian.h>
 #include <string>
 #include <system_error>
+#include <vector>
 
 namespace cybozu {
 
@@ -97,6 +98,17 @@ inline void hton(UInt d, char* p) noexcept {
 }
 
 
+// Tokenize a string by given delimiter.
+// @s  String to be tokenized.
+// @c  Delimiter.
+//
+// This function splits the given string using `c` as a delimiter.
+// The result will not contain the delimiter nor empty strings.
+//
+// @return  A vector of string tokens.
+std::vector<std::string> tokenize(const std::string& s, char c);
+
+
 // Clear memory securely just like memset_s in C11.
 // @s  A pointer to a memory region.
 // @n  The length of the memory region in bytes.
