@@ -54,6 +54,21 @@ void repl_object(const std::vector<repl_socket*>& slaves,
         s->sendv(iov, 4, flush);
 }
 
+void repl_touch(const std::vector<repl_socket*>& slaves,
+                const cybozu::hash_key& key, const object& obj) {
+    char header[BINARY_HEADER_SIZE];
+    fill_header(header, key.length(), 4, 0, binary_command::Touch);
+    char extras[4];
+    cybozu::hton(obj.exptime(), extras);
+    cybozu::tcp_socket::iovec iov[3] = {
+        {header, sizeof(header)},
+        {extras, sizeof(extras)},
+        {key.data(), key.length()}
+    };
+    for( cybozu::tcp_socket* s: slaves )
+        s->sendv(iov, 3, true);
+}
+
 void repl_delete(const std::vector<repl_socket*>& slaves,
                  const cybozu::hash_key& key) {
     char header[BINARY_HEADER_SIZE];
@@ -68,14 +83,12 @@ void repl_delete(const std::vector<repl_socket*>& slaves,
 
 std::size_t repl_recv(const char* p, std::size_t len,
                       cybozu::hash_map<object>& hash) {
-    namespace mc = yrmcds::memcache;
-
     std::size_t consumed = 0;
     while( len > 0 ) {
-        if( ! mc::is_binary_request(p) )
+        if( ! is_binary_request(p) )
             throw std::runtime_error("Invalid replication data");
 
-        mc::binary_request parser(p, len);
+        binary_request parser(p, len);
         std::size_t n = parser.length();
         if( n == 0 ) break;
         p += n;
@@ -88,7 +101,7 @@ std::size_t repl_recv(const char* p, std::size_t len,
         cybozu::hash_map<object>::creator c = nullptr;
 
         switch( parser.command() ) {
-        case mc::binary_command::SetQ:
+        case binary_command::SetQ:
             h = [&parser](const cybozu::hash_key&, object& obj) -> bool {
                 const char* p2;
                 std::size_t len2;
@@ -109,7 +122,18 @@ std::size_t repl_recv(const char* p, std::size_t len,
                                     << std::string(key_data, key_len);
             hash.apply_nolock(cybozu::hash_key(key_data, key_len), h, c);
             break;
-        case mc::binary_command::DeleteQ:
+        case binary_command::Touch:
+            h = [&parser](const cybozu::hash_key&, object& obj) -> bool {
+                ++ g_stats.repl_updated;
+                obj.touch( parser.exptime() );
+                return true;
+            };
+            std::tie(key_data, key_len) = parser.key();
+            cybozu::logger::debug() << "repl: touch "
+                                    << std::string(key_data, key_len);
+            hash.apply_nolock(cybozu::hash_key(key_data, key_len), h, c);
+            break;
+        case binary_command::DeleteQ:
             std::tie(key_data, key_len) = parser.key();
             cybozu::logger::debug() << "repl: remove "
                                     << std::string(key_data, key_len);
