@@ -1,4 +1,4 @@
-#include "../src/memcache.hpp"
+#include "../src/memcache/memcache.hpp"
 
 #include <cybozu/dynbuf.hpp>
 #include <cybozu/tcp.hpp>
@@ -440,6 +440,13 @@ public:
         send(req.data(), req.length());
     }
 
+    void keys(const std::string& key) {
+        request req(binary_command::Keys,
+                    (std::uint16_t)key.size(), key.data(),
+                    0, nullptr, 0, nullptr);
+        send(req.data(), req.length());
+    }
+
     void quit(bool q) {
         request req(q ? binary_command::QuitQ : binary_command::Quit,
                     0, nullptr, 0, nullptr, 0, nullptr);
@@ -1162,21 +1169,113 @@ AUTOTEST(stat_ops) {
     }
 }
 
+AUTOTEST(keys) {
+    client c;
+    response r;
+
+    c.flush_all(true, 0);
+    std::this_thread::sleep_for(std::chrono::seconds(2));
+
+    c.set("foo", "bar", true, 10, 0);
+    c.set("fo1", "bar", true, 10, 0);
+    c.set("zzzzzzzzz", "bar", true, 10, 0);
+
+    c.keys("");
+    int n = 0;
+    while( true ) {
+        cybozu_assert( c.get_response(r) );
+        ASSERT_COMMAND(r, Keys);
+        ASSERT_OK(r);
+        if( std::get<1>(r.key()) == 0 )
+            break;
+        ++n;
+    }
+    cybozu_assert( n == 3 );
+
+    n = 0;
+    c.keys("f");
+    while( true ) {
+        cybozu_assert( c.get_response(r) );
+        ASSERT_COMMAND(r, Keys);
+        ASSERT_OK(r);
+        if( std::get<1>(r.key()) == 0 )
+            break;
+        ++n;
+    }
+    cybozu_assert( n == 2 );
+
+    n = 0;
+    c.keys("foo");
+    while( true ) {
+        cybozu_assert( c.get_response(r) );
+        ASSERT_COMMAND(r, Keys);
+        ASSERT_OK(r);
+        if( std::get<1>(r.key()) == 0 )
+            break;
+        ++n;
+    }
+    cybozu_assert( n == 1 );
+
+    n = 0;
+    c.keys("zzzz");
+    while( true ) {
+        cybozu_assert( c.get_response(r) );
+        ASSERT_COMMAND(r, Keys);
+        ASSERT_OK(r);
+        if( std::get<1>(r.key()) == 0 )
+            break;
+        ++n;
+    }
+    cybozu_assert( n == 1 );
+
+    n = 0;
+    c.keys("b");
+    while( true ) {
+        cybozu_assert( c.get_response(r) );
+        ASSERT_COMMAND(r, Keys);
+        ASSERT_OK(r);
+        if( std::get<1>(r.key()) == 0 )
+            break;
+        ++n;
+    }
+    cybozu_assert( n == 0 );
+}
+
+void print_usage() {
+    std::cout << "Usage: protocol_binary.exe [SERVER [PORT]]\n"
+                 "Environment options:\n"
+                 "  YRMCDS_SERVER : the name of a yrmcds server.\n"
+                 "                  used only when `SERVER` is unspecified.\n"
+                 "  YRMCDS_PORT   : the port number of a yrmcds server.\n"
+                 "                  used only when `PORT` is unspecified.\n"
+              << std::flush;
+}
 
 // main
 bool optparse(int argc, char** argv) {
-    if( argc != 2 && argc != 3 ) {
-        std::cout << "Usage: protocol_binary SERVER [PORT]" << std::endl;
+    if( argc > 3 ) {
+        print_usage();
         return false;
     }
-    g_server = argv[1];
-    if( argc == 3 ) {
-        int n = std::stoi(argv[2]);
-        if( n <= 0 || n > 65535 ) {
-            std::cout << "Invalid port number: " << argv[2] << std::endl;
-            return false;
-        }
-        g_port = n;
+
+    g_server = getenv("YRMCDS_SERVER");
+    const char* env_port = getenv("YRMCDS_PORT");
+    if( env_port != nullptr )
+        g_port = std::stoi(env_port);
+
+    if( argc >= 2 )
+        g_server = argv[1];
+    if( argc >= 3 )
+        g_port = std::stoi(argv[2]);
+
+    if( g_server == nullptr ) {
+        std::cout << "No server specified." << std::endl;
+        print_usage();
+        return false;
+    }
+    if( g_port <= 0 || g_port > 65535 ) {
+        std::cout << "Invalid port number: " << g_port << std::endl;
+        return false;
     }
 
     int s = connect_server();
